/*
   This software is poisoned by GNU GPL copyleft version 1
*/

#include <stdio.h>
#include <cdb.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>

#define FNAME "hash.db"
#define INSERTS 70000
#define DSIZE 40

/* protos */
static void createbench(const char *junk, int opt);
static void rewritebench(const char *junk, size_t junksize,int optimize);
static size_t readbench(size_t howmany,int optimize);
static void deletebench(size_t howmany,int optimize);

int main (int argc, char *argv[])
{
  char *junk;
  int opt;
  char mode;
 
  junk=calloc(1,DSIZE*100);
  if(!junk)
  {
      perror("calloc()");
      exit(1);
  }
 
  /* parse command line arguments */
   
  if(argc>1)
      mode=*argv[1];
  else
      mode='?'; /* must use mode */
  
  if( islower(mode))
  {
      opt=0;
  } else
  {
      opt=1;
      mode=tolower(mode);
  }

  switch(mode)
  {
      case 'c':
               createbench(junk,opt);
	       break;
      case 'o':
               rewritebench(junk,DSIZE,opt);
	       break;
      case 's':
               rewritebench(junk,DSIZE/2,opt);
	       break;
      case 'l':
               rewritebench(junk,DSIZE*2,opt);
	       break;
      case 'r':
              printf("%d records found.\n",readbench(INSERTS,opt));
	      break;
      case 'm':
              readbench(INSERTS*5,opt);
	      break;
      case 'd':
              deletebench(INSERTS,opt);
	      break;	      
      default:
               printf("unknown op. mode: %c \n",mode);
	       printf("Usage: %s mode\n",argv[0]);
	       printf("Supported modes:\n");
	       printf("C - create with optimized 4K block settings\n");
	       printf("c - create with default settings\n");
	       printf("o - overwrite with equal sized data\n");
	       printf("s - overwrite with half sized data\n");
	       printf("l - overwrite with double sized data\n");
	       printf("r - read stored records\n");
	       printf("m - read stored+missing records\n");
	       printf("d - delete records\n");
	       printf("\nNote: mode in uppercase will pass opt flags to db engine.\n");
	       return 2;
  }
  return 0;
}

static void createbench(const char *junk, int optimize)
{
  struct cdb_make cdb;
  int i;
  int fd;
  void * key;

  fd=open(FNAME,O_RDWR|O_CREAT|O_TRUNC,0660);
  if (!fd)
  {
      perror("Can not create database");
      exit(1);
  }
  
  if (cdb_make_start(&cdb, fd))
  {
      perror("cdb make start");
      exit(1);
  }
	 
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(cdb_make_add(&cdb,key,sizeof(int),junk,DSIZE))
      {
	  perror("add");
	  exit(1);
      }
  }

  if (cdb_make_finish(&cdb))
  {
      perror("cdb make finish");
      exit(1);
  }
  close(fd);
  return;
}

static void rewritebench(const char *junk, size_t junksize,int optimize)
{
  struct cdb_make cdb;
  int i;
  int fd;
  void * key;

  fd=open(FNAME,O_RDWR|O_CREAT|O_TRUNC,0660);
  if (!fd)
  {
      perror("Can not create database");
      exit(1);
  }
  
  if (cdb_make_start(&cdb, fd))
  {
      perror("cdb make start");
      exit(1);
  }
	 
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(cdb_make_add(&cdb,key,sizeof(int),junk,junksize))
      {
	  perror("add");
	  exit(1);
      }
  }

  if (cdb_make_finish(&cdb))
  {
      perror("cdb make finish");
      exit(1);
  }
  close(fd);
  return;
}

static size_t readbench(size_t howmany,int optimize)
{
  struct cdb cdb;
  int i;
  size_t records;
  int fd;
  void * key;

  fd=open(FNAME,O_RDONLY,0660);
  if (!fd)
  {
      perror("Can not open database");
      exit(1);
  }

  records=0;
  if (cdb_init(&cdb,fd))
  {
      perror("Can not init cdb");
      exit(1);
  }
  
  for(i=0;i<howmany;i++)
  {
      key=&i;
      if(cdb_find(&cdb,key,sizeof(int))==1)
	  records++;
  }
  cdb_free(&cdb);
  close(fd);
  return records;
}


static void deletebench(size_t howmany,int optimize)
{
  perror("Function not supported");
  exit(1);
  return;
}
