/*
   This software is poisoned by GNU GPL copyleft version 1
*/

#include <sys/types.h>
#include <stdio.h>
#include <db41/db.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <ctype.h>

#define FNAME "hash.db"
#define INSERTS 70000
#define DSIZE 40

/* protos */
static void createbench(const char *junk, int opt);
static void rewritebench(const char *junk, size_t junksize,int optimize);
static size_t readbench(size_t howmany,int optimize);
static void deletebench(size_t howmany,int optimize);

int main (int argc, char *argv[])
{
  char *junk;
  int opt;
  char mode;
 
  junk=calloc(1,DSIZE*100);
  if(!junk)
  {
      perror("calloc()");
      exit(1);
  }
 
  /* parse command line arguments */
   
  if(argc>1)
      mode=*argv[1];
  else
      mode='?'; /* must use mode */
  
  if( islower(mode))
  {
      opt=0;
  } else
  {
      opt=1;
      mode=tolower(mode);
  }

  switch(mode)
  {
      case 'c':
               createbench(junk,opt);
	       break;
      case 'o':
               rewritebench(junk,DSIZE,opt);
	       break;
      case 's':
               rewritebench(junk,DSIZE/2,opt);
	       break;
      case 'l':
               rewritebench(junk,DSIZE*2,opt);
	       break;
      case 'r':
              printf("%d records found.\n",readbench(INSERTS,opt));
	      break;
      case 'm':
              readbench(INSERTS*5,opt);
	      break;
      case 'd':
              deletebench(INSERTS,opt);
	      break;	      
      default:
               printf("unknown op. mode: %c \n",mode);
	       printf("Usage: %s mode\n",argv[0]);
	       printf("Supported modes:\n");
	       printf("C - create with optimized 4K block settings\n");
	       printf("c - create with default settings\n");
	       printf("o - overwrite with equal sized data\n");
	       printf("s - overwrite with half sized data\n");
	       printf("l - overwrite with double sized data\n");
	       printf("r - read stored records\n");
	       printf("m - read stored+missing records\n");
	       printf("d - delete records\n");
	       printf("\nNote: mode in uppercase will pass opt flags to db engine.\n");
	       return 2;
  }
  return 0;
}

static void createbench(const char *junk, int optimize)
{
  DB *db;
  int i;
  DBT key,value;

  if (db_create(&db, NULL, 0)) 
  {
      perror("Can not init database environment");
      exit(1);
  }
 
  if(optimize)
  {
      if (db->set_pagesize(db, 4096))
      {
	  perror("Can not set page size");
	  exit(1);
      }
  }
  if(db->open(db,NULL,FNAME,NULL,DB_HASH,DB_CREATE|DB_TRUNCATE,0660))
  {
      perror("Error opening db.");
      exit(1);
  }

  memset(&key, 0, sizeof(key));
  memset(&value, 0, sizeof(value));

  value.data=junk;
  value.size=DSIZE;
  key.size=sizeof(int);
  
  for(i=0;i<INSERTS;i++)
  {
      key.data=&i;
      if(db->put(db,NULL,&key,&value,0))
      {
	  perror("put");
	  exit(1);
      }
  }

  if(db->close(db,0))
  {
      perror("close");
      exit(1);
  }
  return;
}

static void rewritebench(const char *junk, size_t junksize,int optimize)
{
  DB *db;
  int i;
  DBT key,value;

  if (db_create(&db, NULL, 0)) 
  {
      perror("Can not init database environment");
      exit(1);
  }
 
  if(optimize)
  {
      if (db->set_pagesize(db, 4096))
      {
	  perror("Can not set page size");
	  exit(1);
      }
  }
  if(db->open(db,NULL,FNAME,NULL,DB_HASH,0,0660))
  {
      perror("Error opening db.");
      exit(1);
  }

  memset(&key, 0, sizeof(key));
  memset(&value, 0, sizeof(value));

  value.data=junk;
  value.size=junksize;
  key.size=sizeof(int);
  
  for(i=0;i<INSERTS;i++)
  {
      key.data=&i;
      if(db->put(db,NULL,&key,&value,0))
      {
	  perror("put");
	  exit(1);
      }
  }

  if(db->close(db,0))
  {
      perror("close");
      exit(1);
  }
  return;
}

static size_t readbench(size_t howmany,int optimize)
{
  DB *db;
  int i;
  DBT key,value;
  size_t records;

  records=0;
  if (db_create(&db, NULL, 0)) 
  {
      perror("Can not init database environment");
      exit(1);
  }
 
  if(optimize)
  {
      if (db->set_pagesize(db, 4096))
      {
	  perror("Can not set page size");
	  exit(1);
      }
  }
  if(db->open(db,NULL,FNAME,NULL,DB_HASH,0,0660))
  {
      perror("Error opening db.");
      exit(1);
  }

  memset(&key, 0, sizeof(key));
  memset(&value, 0, sizeof(value));
  
  key.size=sizeof(int);
  
  for(i=0;i<howmany;i++)
  {
      key.data=&i;
      if ( db->get(db,NULL,&key,&value,0) == 0 )
	  records++;
  }

  if(db->close(db,0))
  {
      perror("close");
      exit(1);
  }
  return records;
}


static void deletebench(size_t howmany,int optimize)
{
  DB *db;
  int i;
  DBT key;
  int rc;

  if (db_create(&db, NULL, 0)) 
  {
      perror("Can not init database environment");
      exit(1);
  }
  
  if(db->open(db,NULL,FNAME,NULL,DB_HASH,0,0660))
  {
      perror("Error opening db.");
      exit(1);
  }

  memset(&key, 0, sizeof(key));
  key.size=sizeof(int);
  
  for(i=0;i<howmany;i++)
  {
      key.data=&i;
      rc=db->del(db,NULL,&key,0);
      if(rc<0)
      {
	  perror("delete failed");
	  exit(1);
      }
      else if (rc==1)
	  printf("key %d not found!\n",i);
  }

  if(db->close(db,0))
  {
      perror("close");
      exit(1);
  }
  return;
}
