/*
   This software is poisoned by GNU GPL copyleft version 1
*/

#include <stdio.h>
#include <gdbm.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>

#define FNAME "hash.db"
#define INSERTS 70000
#define DSIZE 40

/* protos */
static void createbench(const char *junk, int opt);
static void rewritebench(const char *junk, size_t junksize,int optimize);
static size_t readbench(size_t howmany,int optimize);
static void deletebench(size_t howmany,int optimize);

int main (int argc, char *argv[])
{
  char *junk;
  int opt;
  char mode;
 
  junk=calloc(1,DSIZE*100);
  if(!junk)
  {
      perror("calloc()");
      exit(1);
  }
 
  /* parse command line arguments */
   
  if(argc>1)
      mode=*argv[1];
  else
      mode='?'; /* must use mode */
  
  if( islower(mode))
  {
      opt=0;
  } else
  {
      opt=1;
      mode=tolower(mode);
  }

  switch(mode)
  {
      case 'c':
               createbench(junk,opt);
	       break;
      case 'o':
               rewritebench(junk,DSIZE,opt);
	       break;
      case 's':
               rewritebench(junk,DSIZE/2,opt);
	       break;
      case 'l':
               rewritebench(junk,DSIZE*2,opt);
	       break;
      case 'r':
              printf("%d records found.\n",readbench(INSERTS,opt));
	      break;
      case 'm':
              readbench(INSERTS*5,opt);
	      break;
      case 'd':
              deletebench(INSERTS,opt);
	      break;	      
      default:
               printf("unknown op. mode: %c \n",mode);
	       printf("Usage: %s mode\n",argv[0]);
	       printf("Supported modes:\n");
	       printf("C - create with optimized 4K block settings\n");
	       printf("c - create with default settings\n");
	       printf("o - overwrite with equal sized data\n");
	       printf("s - overwrite with half sized data\n");
	       printf("l - overwrite with double sized data\n");
	       printf("r - read stored records\n");
	       printf("m - read stored+missing records\n");
	       printf("d - delete records\n");
	       printf("\nNote: mode in uppercase will pass opt flags to db engine.\n");
	       return 2;
  }
  return 0;
}

static void createbench(const char *junk, int optimize)
{
  GDBM_FILE db;
  int i;
  datum key,value;

  if(optimize)
      db=gdbm_open(FNAME,4096,GDBM_NEWDB,0640,NULL);
  else
      db=gdbm_open(FNAME,0,GDBM_NEWDB,0640,NULL);
  if (!db)
  {
      perror("Can not create database");
      exit(1);
  }
  
  value.dptr=junk;
  value.dsize=DSIZE;
  key.dsize=sizeof(int);
  
  for(i=0;i<INSERTS;i++)
  {
      key.dptr=&i;
      if(gdbm_store(db,key,value,GDBM_REPLACE))
      {
	  perror("insert");
	  exit(1);
      }
  }
  gdbm_close(db);
  return;
}

static void rewritebench(const char *junk, size_t junksize,int optimize)
{
  GDBM_FILE db;
  int i;
  datum key,value;

  db=gdbm_open(FNAME,0,GDBM_WRITER,0640,NULL);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  value.dptr=junk;
  value.dsize=junksize;
  key.dsize=sizeof(int);
  
  for(i=0;i<INSERTS;i++)
  {
      key.dptr=&i;
      if(gdbm_store(db,key,value,GDBM_REPLACE))
      {
	  perror("replace");
	  exit(1);
      }
  }
  gdbm_close(db);
  return;
}

static size_t readbench(size_t howmany,int optimize)
{
  GDBM_FILE db;
  int i;
  datum key,value;
  size_t records;

  records=0;
  db=gdbm_open(FNAME,0,GDBM_READER,0660,NULL);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  key.dsize=sizeof(int);
  
  for(i=0;i<howmany;i++)
  {
      key.dptr=&i;
      value=gdbm_fetch(db,key);
      if(value.dptr != NULL)
      {
	  records++;
	  free(value.dptr);
      }
  }
  gdbm_close(db);
  return records;
}


static void deletebench(size_t howmany,int optimize)
{
  GDBM_FILE db;
  int i;
  datum key;
  int rc;

  db=gdbm_open(FNAME,0,GDBM_WRITER,0640,NULL);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  key.dsize=sizeof(int);
  
  for(i=0;i<howmany;i++)
  {
      key.dptr=&i;
      rc=gdbm_delete(db,key);
      if(rc<0)
      {
	  perror("delete failed");
	  exit(1);
      }
  }

  gdbm_close(db);
  return;
}
