/*
   This software is poisoned by GNU GPL copyleft version 1
*/

#include <stdio.h>
#include <fcntl.h>
#include <puredb_read.h>
#include <puredb_write.h>
#include <stdlib.h>
#include <ctype.h>

#define FNAME "hash.db"
#define INSERTS 70000
#define DSIZE 40

/* protos */
static void createbench(const char *junk, int opt);
static void rewritebench(const char *junk, size_t junksize,int optimize);
static size_t readbench(size_t howmany,int optimize);
static void deletebench(size_t howmany,int optimize);

int main (int argc, char *argv[])
{
  char *junk;
  int opt;
  char mode;
 
  junk=calloc(1,DSIZE*100);
  if(!junk)
  {
      perror("calloc()");
      exit(1);
  }
 
  /* parse command line arguments */
   
  if(argc>1)
      mode=*argv[1];
  else
      mode='?'; /* must use mode */
  
  if( islower(mode))
  {
      opt=0;
  } else
  {
      opt=1;
      mode=tolower(mode);
  }

  switch(mode)
  {
      case 'c':
               createbench(junk,opt);
	       break;
      case 'o':
               rewritebench(junk,DSIZE,opt);
	       break;
      case 's':
               rewritebench(junk,DSIZE/2,opt);
	       break;
      case 'l':
               rewritebench(junk,DSIZE*2,opt);
	       break;
      case 'r':
              printf("%d records found.\n",readbench(INSERTS,opt));
	      break;
      case 'm':
              readbench(INSERTS*5,opt);
	      break;
      case 'd':
              deletebench(INSERTS,opt);
	      break;	      
      default:
               printf("unknown op. mode: %c \n",mode);
	       printf("Usage: %s mode\n",argv[0]);
	       printf("Supported modes:\n");
	       printf("C - create with optimized 4K block settings\n");
	       printf("c - create with default settings\n");
	       printf("o - overwrite with equal sized data\n");
	       printf("s - overwrite with half sized data\n");
	       printf("l - overwrite with double sized data\n");
	       printf("r - read stored records\n");
	       printf("m - read stored+missing records\n");
	       printf("d - delete records\n");
	       printf("\nNote: mode in uppercase will pass opt flags to db engine.\n");
	       return 2;
  }
  return 0;
}

static void createbench(const char *junk, int optimize)
{
  PureDBW db;
  int i;
  char *key, *value;

  if (puredbw_open(&db,"index.db","data.db",FNAME))
  {
      perror("Can not create database");
      exit(1);
  }
  
  value=junk;
  
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(puredbw_add(&db,key,sizeof(int),value,DSIZE))
      {
	  perror("add");
	  exit(1);
      }
  }
  if(puredbw_close(&db))
  {
      perror("close");
      exit(1);
  }
  puredbw_free(&db);
  return;
}

static void rewritebench(const char *junk, size_t junksize,int optimize)
{
  PureDBW db;
  int i;
  char *key, *value;

  if (puredbw_open(&db,"index.db","data.db",FNAME))
  {
      perror("Can not create database");
      exit(1);
  }
  
  value=junk;
  
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(puredbw_add(&db,key,sizeof(int),value,junksize))
      {
	  perror("add");
	  exit(1);
      }
  }
  if(puredbw_close(&db))
  {
      perror("close");
      exit(1);
  }
  puredbw_free(&db);
  return;
}

static size_t readbench(size_t howmany,int optimize)
{
  PureDB db;
  int i;
  size_t records;
  char *key;
  off_t pos;
  size_t len;

  records=0;
  if (puredb_open(&db,FNAME))
  {
      perror("Can not open database");
      exit(1);
  }
  
  for(i=0;i<howmany;i++)
  {
      key=&i;
      if(puredb_find(&db,key,sizeof(int),&pos,&len)==0)
      {
	  void *data;
	  records++;
	  /* read the record */
	  data=puredb_read(&db,pos,len);
	  puredb_read_free(data);
	  }
  }

  if(puredb_close(&db))
  {
      perror("close");
      exit(1);
  }
  return records;
}


static void deletebench(size_t howmany,int optimize)
{
  perror("funtion not supported");
  exit(1);   
  return;
}
