/*
   This software is poisoned by GNU GPL copyleft version 1
*/

#include <stdio.h>
#include <depot.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>

#define FNAME "hash.db"
#define INSERTS 70000
#define DSIZE 40

/* protos */
static void createbench(const char *junk, int opt);
static void rewritebench(const char *junk, size_t junksize,int optimize);
static size_t readbench(size_t howmany,int optimize);
static void deletebench(size_t howmany,int optimize);

int main (int argc, char *argv[])
{
  char *junk;
  int opt;
  char mode;
 
  junk=calloc(1,DSIZE*100);
  if(!junk)
  {
      perror("calloc()");
      exit(1);
  }
 
  /* parse command line arguments */
   
  if(argc>1)
      mode=*argv[1];
  else
      mode='?'; /* must use mode */
  
  if( islower(mode))
  {
      opt=0;
  } else
  {
      opt=1;
      mode=tolower(mode);
  }

  switch(mode)
  {
      case 'c':
               createbench(junk,opt);
	       break;
      case 'o':
               rewritebench(junk,DSIZE,opt);
	       break;
      case 's':
               rewritebench(junk,DSIZE/2,opt);
	       break;
      case 'l':
               rewritebench(junk,DSIZE*2,opt);
	       break;
      case 'r':
              printf("%d records found.\n",readbench(INSERTS,opt));
	      break;
      case 'm':
              readbench(INSERTS*5,opt);
	      break;
      case 'd':
              deletebench(INSERTS,opt);
	      break;	      
      default:
               printf("unknown op. mode: %c \n",mode);
	       printf("Usage: %s mode\n",argv[0]);
	       printf("Supported modes:\n");
	       printf("C - create with optimized 4K block settings\n");
	       printf("c - create with default settings\n");
	       printf("o - overwrite with equal sized data\n");
	       printf("s - overwrite with half sized data\n");
	       printf("l - overwrite with double sized data\n");
	       printf("r - read stored records\n");
	       printf("m - read stored+missing records\n");
	       printf("d - delete records\n");
	       printf("\nNote: mode in uppercase will pass opt flags to db engine.\n");
	       return 2;
  }
  return 0;
}

static void createbench(const char *junk, int optimize)
{
  DEPOT *db;
  int i;
  char *key,*value;

  if(optimize)
    db=dpopen(FNAME,DP_OWRITER|DP_OTRUNC|DP_OCREAT,4096);
  else
    db=dpopen(FNAME,DP_OWRITER|DP_OTRUNC|DP_OCREAT,0);
  if (!db)
  {
      perror("Can not create database");
      exit(1);
  }
  
  value=junk;
  
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(!dpput(db,key,sizeof(key),value,DSIZE,DP_DOVER))
      {
	  perror("put");
	  exit(1);
      }
  }
  if(!dpclose(db))
  {
      perror("close");
      exit(1);
  }
  return;
}

static void rewritebench(const char *junk, size_t junksize,int optimize)
{
  DEPOT *db;
  int i;
  char *key,*value;

  db=dpopen(FNAME,DP_OWRITER,0);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  value=junk;
  
  for(i=0;i<INSERTS;i++)
  {
      key=&i;
      if(!dpput(db,key,sizeof(key),value,junksize,DP_DOVER))
      {
	  perror("put");
	  exit(1);
      }
  }
  if(!dpclose(db))
  {
      perror("close");
      exit(1);
  }
  return;
}

static size_t readbench(size_t howmany,int optimize)
{
  DEPOT *db;
  int i;
  char *key,*value;
  size_t records;

  records=0;
  db=dpopen(FNAME,DP_OREADER,0);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  for(i=0;i<howmany;i++)
  {
      key=&i;
      value=dpget(db,key,sizeof(int),0,-1,NULL);
      if(value != NULL)
      {
	  records++;
	  free(value);
      }
  }
  if(!dpclose(db))
  {
      perror("close");
      exit(1);
  }
  return records;
}


static void deletebench(size_t howmany,int optimize)
{
  DEPOT *db;
  int i;
  char *key;
  int rc;

  db=dpopen(FNAME,DP_OWRITER,0);
  if (!db)
  {
      perror("Can not open database");
      exit(1);
  }
  
  for(i=0;i<howmany;i++)
  {
      key=&i;
      rc=dpout(db,key,sizeof(int));
      if( ! rc)
      {
	  perror("delete failed");
	  exit(1);
      }
  }

  if(!dpclose(db))
  {
      perror("close");
      exit(1);
  }
  return;
}
